package com.sd.business.service.order.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.file.utils.ObsFileUtil;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.system.utils.UserUtil;
import com.sd.business.entity.bom.bo.BomSpecBo;
import com.sd.business.entity.bom.constant.BomClassifyConstant;
import com.sd.business.entity.department.constant.DepartmentConstant;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.in.dto.InOutStorageDto;
import com.sd.business.entity.in.emums.InDetailTypeEnum;
import com.sd.business.entity.in.emums.InOutTypeEnum;
import com.sd.business.entity.in.emums.OutDetailTypeEnum;
import com.sd.business.entity.in.po.InOutStorageBom;
import com.sd.business.entity.order.bo.OrderExchangeAccountBo;
import com.sd.business.entity.order.dto.OrderExchangeDto;
import com.sd.business.entity.order.dto.OrderExchangeSelectDto;
import com.sd.business.entity.order.po.*;
import com.sd.business.entity.order.vo.OrderExchangeDetailVo;
import com.sd.business.entity.order.vo.OrderExchangeVo;
import com.sd.business.entity.statement.po.StatementOfAccount;
import com.sd.business.entity.warehouse.constant.WarehouseConstant;
import com.sd.business.mapper.order.OrderExchangeMapper;
import com.sd.business.service.in.InOutStorageService;
import com.sd.business.service.inventory.InventoryFinishedService;
import com.sd.business.service.order.*;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.business.service.statement.StatementOfAccountService;
import com.sd.business.util.CodeEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * <p>
 * 订单退换货 服务实现类
 * </p>
 *
 * @author
 * @since 2023-09-14
 */
@Service
public class OrderExchangeServiceImpl extends ServiceImpl<OrderExchangeMapper, OrderExchange> implements OrderExchangeService {

    @Autowired
    private OrderExchangeDetailService orderExchangeDetailService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderSkuBomService orderSkuBomService;

    @Autowired
    private StatementOfAccountService statementOfAccountService;

    @Autowired
    private InOutStorageService inOutStorageService;

    @Autowired
    private InventoryFinishedService inventoryFinishedService;

    @Override
    public Page<OrderExchangeVo> getPage(OrderExchangeSelectDto dto) {
        IWrapper<OrderExchange> wrapper = getWrapper();
        wrapper.orderByDesc("oe", OrderExchange::getId);
        wrapper.eq("oe", OrderExchange::getOrderInfoId, dto.getOrderInfoId());
        wrapper.eq("oe", OrderExchange::getStatus, dto.getStatus());
        wrapper.like("oe", OrderExchange::getCode, dto.getCode());
        wrapper.like("oi", OrderInfo::getCode, dto.getOrderCode());
        wrapper.like("oi", OrderInfo::getWlnCode, dto.getOrderWlnCode());
        wrapper.eq("d", Department::getId, dto.getDepartmentId());

        Page<OrderExchangeVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<OrderExchangeVo> records = page.getRecords();

        if (records.isEmpty()) {
            return page;
        }

        // 赋值操作人名称
        UserUtil.assignmentNickName(records, BasePo::getCreateUser, OrderExchangeVo::setCreateUserName);

        List<Long> idList = records.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        List<OrderExchangeDetail> list = orderExchangeDetailService.list(q -> q.in(OrderExchangeDetail::getOrderExchangeId, idList));
        List<OrderExchangeDetailVo> orderExchangeDetailList = BeanUtil.copyToList(list, OrderExchangeDetailVo.class);

        // 赋值sku规格id和采购数量
        orderSkuService.attributeAssign(orderExchangeDetailList, OrderExchangeDetailVo::getOrderSkuId, (item, orderSku) -> {
            item.setSkuSpecId(orderSku.getSkuSpecId());
            item.setBuyQuantity(orderSku.getQuantity());
        });

        // 赋值sku品号品名
        skuSpecService.attributeAssign(orderExchangeDetailList, OrderExchangeDetailVo::getSkuSpecId, (item, skuSpec) -> {
            item.setSkuSpecCode(skuSpec.getCode());
            item.setSkuSpecName(skuSpec.getName());
        });

        Map<Long, List<OrderExchangeDetailVo>> map = orderExchangeDetailList.stream()
                .collect(Collectors.groupingBy(OrderExchangeDetailVo::getOrderExchangeId));

        for (OrderExchangeVo record : records) {
            record.setOrderExchangeDetailList(map.getOrDefault(record.getId(), Collections.emptyList()));
        }

        return page;
    }

    @Override
    public OrderExchangeVo detail(Long id) {
        OrderExchange orderExchange = this.getById(id);
        if (orderExchange == null) {
            throw new ServiceException("未知订单号");
        }

        OrderExchangeVo result = BeanUtil.toBean(orderExchange, OrderExchangeVo.class);
        List<OrderExchangeDetail> list = orderExchangeDetailService.list(q -> q.eq(OrderExchangeDetail::getOrderExchangeId, id));
        List<OrderExchangeDetailVo> orderExchangeDetailList = BeanUtil.copyToList(list, OrderExchangeDetailVo.class);

        // 赋值sku规格id和采购数量
        orderSkuService.attributeAssign(orderExchangeDetailList, OrderExchangeDetailVo::getOrderSkuId, (item, orderSku) -> {
            item.setSkuSpecId(orderSku.getSkuSpecId());
            item.setBuyQuantity(orderSku.getQuantity());
        });

        // 赋值sku品号品名
        skuSpecService.attributeAssign(orderExchangeDetailList, OrderExchangeDetailVo::getSkuSpecId, (item, skuSpec) -> {
            item.setSkuSpecCode(skuSpec.getCode());
            item.setSkuSpecName(skuSpec.getName());
        });

        result.setOrderExchangeDetailList(orderExchangeDetailList);

        return result;
    }

    @DSTransactional
    @Override
    public void add(OrderExchangeDto orderExchangeDto) {
        orderExchangeDto.setId(IdWorker.getId());

        List<OrderExchangeDetail> orderExchangeDetailList = orderExchangeDto.getOrderExchangeDetailList();

        // 获取订单sku id集合
        List<Long> orderSkuIdList = orderExchangeDetailList.stream()
                .map(OrderExchangeDetail::getOrderSkuId)
                .collect(Collectors.toList());

        // 根据集合查询订单数量
        Map<Long, OrderSku> map = orderSkuService.mapKEntity(BaseIdPo::getId,
                q -> q.in(BaseIdPo::getId, orderSkuIdList));
        List<OrderSkuBom> orderSkuBomList = orderSkuBomService.list(q -> q.in(OrderSkuBom::getOrderSkuId, orderSkuIdList));
        // 筛选掉可二次利用的包材
        List<Long> bomSpecIdList = orderSkuBomList.stream()
                .map(OrderSkuBom::getBomSpecId)
                .distinct()
                .collect(Collectors.toList());
        Map<Long, BomSpecBo> bomSpecBoMap = skuSpecService.getBomSpecBoByIdList(bomSpecIdList);
        List<OrderSkuBom> tempOrderSkuBomList = orderSkuBomList.stream()
                .filter(item -> {
                    BomSpecBo bomSpecBo = bomSpecBoMap.get(item.getBomSpecId());
                    if (bomSpecBo == null
                            || !(Objects.equals(bomSpecBo.getClassifyId(), BomClassifyConstant.MESH_BAG)
                                || Objects.equals(bomSpecBo.getClassifyId(), BomClassifyConstant.SUSPENDERS))) {
                        return false;
                    }
                    return true;
                })
                .collect(Collectors.toList());
        Map<Long, List<OrderSkuBom>> orderSkuBomMap = tempOrderSkuBomList.stream().collect(Collectors.groupingBy(OrderSkuBom::getOrderSkuId));

        // 已添加订单产品退货列表
        List<OrderExchangeDetail> list = new ArrayList<>();
        List<OrderSku> tempOrderSkuList = new ArrayList<>();
        if (orderExchangeDto.getType().equals(1)) {
            list = orderExchangeDetailService.list(q -> q.in(OrderExchangeDetail::getOrderSkuId, orderSkuIdList));
        }

        for (OrderExchangeDetail orderExchangeDetail : orderExchangeDetailList) {

            Long orderSkuId = orderExchangeDetail.getOrderSkuId();

            // 数量为空，没找到订单sku id
            OrderSku orderSku = map.get(orderSkuId);
            if (orderSku == null) {
                throw new ServiceException("未知订单skuId：" + orderSkuId);
            }
            BigDecimal quantity = orderSku.getQuantity();

            // 退换货数量大于订单产品购买数量
            if (orderExchangeDetail.getQuantity().compareTo(quantity) > 0) {
                throw new ServiceException("退换货数量不能大于订单产品购买数量");
            }

            // 如果退货，已退货数量+本次退货数量不能大于订单产品数量
            if (orderExchangeDto.getType().equals(1)) {
                BigDecimal returnQuantity = list.stream()
                        .filter(item -> Objects.equals(item.getOrderSkuId(), orderSkuId))
                        .map(OrderExchangeDetail::getQuantity)
                        .reduce(BigDecimal.ZERO, BigDecimal::add);

                if (returnQuantity.add(orderExchangeDetail.getQuantity()).compareTo(quantity) > 0) {
                    throw new ServiceException("已记录退货数量加本次退货数量大于订单产品购买数量");
                }
            }

            orderExchangeDetail.setOrderExchangeId(orderExchangeDto.getId());
            orderExchangeDetail.setReturnStatus(StatusConstant.NO);
            orderExchangeDetail.setExchangeStatus(StatusConstant.NO);

            BigDecimal packagingMaterialCost = orderSkuBomMap.getOrDefault(orderSkuId, Collections.emptyList()).stream()
                    .map(item -> item.getUnitPrice().multiply(item.getQuantity()))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            orderExchangeDetail.setReturnAmount((orderSku.getUnitPrice()
                    .add(packagingMaterialCost)).multiply(orderExchangeDetail.getQuantity()));

            // 保存临时入库数据
            OrderSku tempOrderSku = new OrderSku();
            tempOrderSku.setId(orderSku.getId());
            tempOrderSku.setSkuSpecId(orderSku.getSkuSpecId());
            tempOrderSku.setBomSpecId(orderSku.getBomSpecId());
            tempOrderSku.setQuantity(orderExchangeDetail.getQuantity());
            tempOrderSkuList.add(tempOrderSku);
        }

        orderExchangeDto.setCode(orderExchangeDto.getType() == 1 ? CodeEnum.TH_CODE.getCode() : CodeEnum.HH_CODE.getCode());
        orderExchangeDto.setStatus(StatusConstant.YES);

        OrderInfo orderInfo = orderService.getById(orderExchangeDto.getOrderInfoId());
        // 获取对账单
        StatementOfAccount statementOfAccount = this.getStatementOfAccount(orderInfo.getDepartmentId());
        orderExchangeDto.setStatementOfAccountId(statementOfAccount.getId());

        // 售后商品入库
        if (Objects.equals(orderExchangeDto.getWarehouseId(), WarehouseConstant.PRODUCTION_DEFECTIVE)) {
            List<InOutStorageBom> tempInOutStorageBomList = tempOrderSkuList.stream().map(item -> {
                InOutStorageBom inStorageBom = new InOutStorageBom();
                inStorageBom.setBomSpecId(item.getBomSpecId());
                inStorageBom.setQuantity(item.getQuantity());
                return inStorageBom;
            }).collect(Collectors.toList());

            // 合并相同bom规格出库数量
            List<InOutStorageBom> inOutStorageBomList = new ArrayList<>(tempInOutStorageBomList.stream()
                    .collect(Collectors.toMap(
                        InOutStorageBom::getBomSpecId,
                        Function.identity(),
                        (v1, v2) -> {
                            v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                            return v1;
                        })).values());

            InOutStorageDto inOutStorageDto = new InOutStorageDto();
            inOutStorageDto.setType(InOutTypeEnum.IN.getKey());
            inOutStorageDto.setDetailType(InDetailTypeEnum.ABANDON.getKey());
            inOutStorageDto.setWarehouseId(orderExchangeDto.getWarehouseId());
            inOutStorageDto.setDepartmentId(DepartmentConstant.SD_SPORTS);
            inOutStorageDto.setApplicant(SecurityUtils.getLoginUser().getUser().getNickName());
            inOutStorageDto.setRemark("订单：" + orderInfo.getCode() + "订单售后报废入库");
            inOutStorageDto.setInOutStorageBomList(inOutStorageBomList);
            inOutStorageService.add(inOutStorageDto);
        } else {
            // 成品质检通过后出库包材，重新包装
            Map<Long, OrderSku> finishedOrderSkuMap = tempOrderSkuList.stream().collect(Collectors.toMap(BaseIdPo::getId, Function.identity()));
            orderSkuBomList.removeAll(tempOrderSkuBomList);
            List<InOutStorageBom> outStorageBomList = orderSkuBomList.stream().map(item -> {
                InOutStorageBom inOutStorageBom = new InOutStorageBom();
                inOutStorageBom.setBomSpecId(item.getBomSpecId());
                inOutStorageBom.setQuantity(item.getQuantity().multiply(finishedOrderSkuMap.get(item.getOrderSkuId()).getQuantity()));
                return inOutStorageBom;
            }).collect(Collectors.toList());
            InOutStorageDto inOutStorageDto = new InOutStorageDto();
            inOutStorageDto.setType(InOutTypeEnum.OUT.getKey());
            inOutStorageDto.setDetailType(OutDetailTypeEnum.PRODUCTION.getKey());
            inOutStorageDto.setWarehouseId(WarehouseConstant.PACKAGING_MATERIAL);
            inOutStorageDto.setDepartmentId(DepartmentConstant.SD_SPORTS);
            inOutStorageDto.setApplicant(SecurityUtils.getLoginUser().getUser().getNickName());
            inOutStorageDto.setInOutStorageBomList(outStorageBomList);
            inOutStorageDto.setLockStorage(StatusConstant.NO);
            inOutStorageService.add(inOutStorageDto);
            // 入成品库
            inventoryFinishedService.noSourceInWarehousing(tempOrderSkuList);
        }

        // 保存数据
        save(orderExchangeDto);
        orderExchangeDetailService.saveBatch(orderExchangeDetailList);
        ObsFileUtil.saveFile(orderExchangeDto.getFileList(), orderExchangeDto.getId());
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void orderExchangeCheck(OrderExchangeDto orderExchangeDto) {
        OrderExchange orderExchange = this.getById(orderExchangeDto.getId());
        if (orderExchange == null) {
            throw new ServiceException("未知订单号");
        }
        if (Objects.equals(orderExchange.getStatus(), StatusConstant.YES)) {
            throw new ServiceException("订单已质检");
        }
        orderExchange.setStatus(StatusConstant.YES);
        List<OrderExchangeDetail> detailList = orderExchangeDto.getOrderExchangeDetailList();
        if (ObjectUtil.isEmpty(detailList)) {
            throw new ServiceException("订单商品不能为空");
        }
        OrderInfo orderInfo = orderService.getById(orderExchange.getOrderInfoId());
        List<OrderSku> tempOrderSkuList = new ArrayList<>();
        List<InOutStorageBom> tempInOutStorageBomList = new ArrayList<>();
        List<Long> orderSkuIds = detailList.stream().map(OrderExchangeDetail::getOrderSkuId).collect(Collectors.toList());
        Map<Long, OrderSku> orderSkuMap = orderSkuService.mapKEntity(BaseIdPo::getId, q -> q.in(BaseIdPo::getId, orderSkuIds));
        for (OrderExchangeDetail detail : detailList) {
            int compareTo = detail.getQuantity().compareTo(detail.getCheckPassesQuantity());
            if (compareTo == 0) {
                // 质检数量和退货数量一致时，全部入库成品仓
                OrderSku orderSku = new OrderSku();
                orderSku.setId(detail.getOrderSkuId());
                orderSku.setSkuSpecId(orderSkuMap.get(detail.getOrderSkuId()).getSkuSpecId());
                orderSku.setQuantity(detail.getQuantity());
                tempOrderSkuList.add(orderSku);
            } else if (compareTo > 0) {
                // 质检数量和退货数量不一致时，质检通过数量入库成品仓，其它入次品仓
                OrderSku orderSku = orderSkuMap.get(detail.getOrderSkuId());
                OrderSku tempOrderSku = new OrderSku();
                tempOrderSku.setId(orderSku.getId());
                tempOrderSku.setSkuSpecId(orderSku.getSkuSpecId());
                tempOrderSku.setQuantity(detail.getCheckPassesQuantity());
                tempOrderSkuList.add(tempOrderSku);
                InOutStorageBom inStorageBom = new InOutStorageBom();
                inStorageBom.setBomSpecId(orderSku.getBomSpecId());
                inStorageBom.setQuantity(detail.getQuantity().subtract(detail.getCheckPassesQuantity()));
                tempInOutStorageBomList.add(inStorageBom);
            } else {
                throw new ServiceException("质检数量不能超过退货数量！");
            }
        }

        // 入库
        if (!tempOrderSkuList.isEmpty()) {
            // 成品质检通过后出库包材，重新包装
            Map<Long, OrderSku> finishedOrderSkuMap = tempOrderSkuList.stream().collect(Collectors.toMap(BaseIdPo::getId, Function.identity()));
            List<Long> orderSkuId = tempOrderSkuList.stream().map(BaseIdPo::getId).collect(Collectors.toList());
            List<OrderSkuBom> orderSkuBomList = orderSkuBomService.list(q -> q.in(OrderSkuBom::getOrderSkuId, orderSkuId));
            // 筛选掉可二次利用的包材
            List<Long> bomSpecIdList = orderSkuBomList.stream()
                    .map(OrderSkuBom::getBomSpecId)
                    .distinct()
                    .collect(Collectors.toList());
            Map<Long, BomSpecBo> bomSpecBoMap = skuSpecService.getBomSpecBoByIdList(bomSpecIdList);
            List<OrderSkuBom> tempOrderSkuBomList = orderSkuBomList.stream()
                    .filter(item -> {
                        BomSpecBo bomSpecBo = bomSpecBoMap.get(item.getBomSpecId());
                        if (bomSpecBo == null
                                || Objects.equals(bomSpecBo.getClassifyId(), BomClassifyConstant.MESH_BAG)
                                    || Objects.equals(bomSpecBo.getClassifyId(), BomClassifyConstant.SUSPENDERS)) {
                            return false;
                        }
                        return true;
                    })
                    .collect(Collectors.toList());
            if (!tempOrderSkuBomList.isEmpty()) {
                List<InOutStorageBom> outStorageBomList = tempOrderSkuBomList.stream().map(item -> {
                    InOutStorageBom inOutStorageBom = new InOutStorageBom();
                    inOutStorageBom.setBomSpecId(item.getBomSpecId());
                    inOutStorageBom.setQuantity(item.getQuantity().multiply(finishedOrderSkuMap.get(item.getOrderSkuId()).getQuantity()));
                    return inOutStorageBom;
                }).collect(Collectors.toList());
                InOutStorageDto inOutStorageDto = new InOutStorageDto();
                inOutStorageDto.setType(InOutTypeEnum.OUT.getKey());
                inOutStorageDto.setDetailType(OutDetailTypeEnum.PRODUCTION.getKey());
                inOutStorageDto.setWarehouseId(WarehouseConstant.PACKAGING_MATERIAL);
                inOutStorageDto.setDepartmentId(DepartmentConstant.SD_SPORTS);
                inOutStorageDto.setApplicant(SecurityUtils.getLoginUser().getUser().getNickName());
                inOutStorageDto.setInOutStorageBomList(outStorageBomList);
                inOutStorageDto.setLockStorage(StatusConstant.NO);
                inOutStorageService.add(inOutStorageDto);
            }

            inventoryFinishedService.noSourceInWarehousing(tempOrderSkuList);
        }
        if (!tempInOutStorageBomList.isEmpty()) {
            // 合并相同bom规格出库数量
            List<InOutStorageBom> inOutStorageBomList = new ArrayList<>(tempInOutStorageBomList.stream()
                    .collect(Collectors.toMap(
                            InOutStorageBom::getBomSpecId,
                            Function.identity(),
                            (v1, v2) -> {
                                v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                                return v1;
                            })).values());

            InOutStorageDto inOutStorageDto = new InOutStorageDto();
            inOutStorageDto.setType(InOutTypeEnum.IN.getKey());
            inOutStorageDto.setDetailType(InDetailTypeEnum.ABANDON.getKey());
            inOutStorageDto.setWarehouseId(WarehouseConstant.PRODUCTION_DEFECTIVE);
            inOutStorageDto.setDepartmentId(DepartmentConstant.SD_SPORTS);
            inOutStorageDto.setApplicant(SecurityUtils.getLoginUser().getUser().getNickName());
            inOutStorageDto.setRemark("订单：" + orderInfo.getCode() + "订单售后报废入库");
            inOutStorageDto.setInOutStorageBomList(inOutStorageBomList);
            inOutStorageService.add(inOutStorageDto);
        }
        // 获取对账单
        StatementOfAccount statementOfAccount = this.getStatementOfAccount(orderInfo.getDepartmentId());
        orderExchange.setStatementOfAccountId(statementOfAccount.getId());

        // 更新数据
        this.updateById(orderExchange);
        orderExchangeDetailService.updateBatchById(detailList);

    }

    @Override
    public Map<Long, OrderExchangeAccountBo> orderExchangeAccountStatistics(List<Long> statementOfAccountIdList) {
        IWrapper<OrderExchange> wrapper = getWrapper();
        wrapper.eq("oe", OrderExchange::getStatus, StatusConstant.YES);
        wrapper.in("oe", OrderExchange::getStatementOfAccountId, statementOfAccountIdList);
        wrapper.groupBy("oe.statement_of_account_id");

        List<OrderExchangeAccountBo> list = this.baseMapper.orderExchangeAccountStatistics(wrapper);

        if (list.isEmpty()) {
            return Collections.emptyMap();
        }

        return list.stream().collect(Collectors.toMap(OrderExchangeAccountBo::getStatementOfAccountId, Function.identity()));
    }

    /**
     * 获取对账单
     * @param departmentId
     * @return
     */
    private StatementOfAccount getStatementOfAccount(Long departmentId) {
        Date date = new Date();
        Date beginDate = DateUtil.beginOfDay(date);
        Date endDate = DateUtil.endOfDay(date);
        StatementOfAccount statementOfAccount = statementOfAccountService.getOne(q -> q
                .eq(StatementOfAccount::getDepartmentId, departmentId)
                .between(StatementOfAccount::getTimePeriod, beginDate, endDate));

        if (statementOfAccount == null) {
            statementOfAccount = new StatementOfAccount();
            statementOfAccount.setDepartmentId(departmentId);
            statementOfAccount.setCode(CodeEnum.STATEMENT_OF_ACCOUNT_CODE.getCode());
            statementOfAccount.setType(2);
            statementOfAccount.setTimePeriod(new Date());
            statementOfAccountService.save(statementOfAccount);
        }
        return statementOfAccount;
    }
}
