package com.sd.business.service.statement.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.file.entity.FileInfoVo;
import com.fjhx.file.utils.ObsFileUtil;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.framework.config.ThreadPoolConfig;
import com.sd.business.entity.bom.bo.BomSpecBo;
import com.sd.business.entity.bom.constant.BomClassifyConstant;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.excel.enums.ExcelTypeEnum;
import com.sd.business.entity.order.bo.OrderExchangeAccountBo;
import com.sd.business.entity.order.enums.OrderClassifyEnum;
import com.sd.business.entity.order.po.*;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.entity.statement.dto.FileUploadDto;
import com.sd.business.entity.statement.dto.StatementOfAccountDto;
import com.sd.business.entity.statement.dto.StatementOfAccountSelectDto;
import com.sd.business.entity.statement.po.StatementOfAccount;
import com.sd.business.entity.statement.vo.*;
import com.sd.business.mapper.statement.StatementOfAccountMapper;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.department.DepartmentService;
import com.sd.business.service.excel.ExcelGenerateLogService;
import com.sd.business.service.order.*;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.business.service.statement.StatementOfAccountExportService;
import com.sd.business.service.statement.StatementOfAccountService;
import com.sd.business.strategy.impl.DocumentByOrderExcelExportStrategy;
import com.sd.business.util.CodeEnum;
import com.sd.framework.util.Assert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.function.Function;
import java.util.stream.Collectors;

import static java.util.Comparator.comparing;


/**
 * <p>
 * 对账单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-31
 */
@Service
public class StatementOfAccountServiceImpl extends ServiceImpl<StatementOfAccountMapper, StatementOfAccount> implements StatementOfAccountService {

    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private OrderSkuBomService orderSkuBomService;

    @Autowired
    private DepartmentService departmentService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private BomSpecService bomSpecService;

    @Autowired
    private StatementOfAccountExportService statementOfAccountExportService;

    @Qualifier(ThreadPoolConfig.threadPoolTaskExecutor)
    @Autowired
    private ThreadPoolExecutor threadPoolExecutor;

    @Autowired
    private ExcelGenerateLogService excelGenerateLogService;

    @Autowired
    private OrderExchangeService orderExchangeService;

    @Autowired
    private OrderExchangeDetailService orderExchangeDetailService;

    @Override
    public Page<StatementOfAccountVo> getPage(StatementOfAccountSelectDto dto) {

        IWrapper<StatementOfAccount> wrapper = getWrapper();
        wrapper.like("soa", StatementOfAccount::getCode, dto.getCode());
        wrapper.eq("soa", StatementOfAccount::getDepartmentId, dto.getDepartmentId());
        wrapper.ge("soa", StatementOfAccount::getTimePeriod, dto.getBeginTime());
        wrapper.le("soa", StatementOfAccount::getTimePeriod, dto.getEndTime());
        wrapper.orderByDesc("soa", StatementOfAccount::getCode);

        Page<StatementOfAccountVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<StatementOfAccountVo> records = page.getRecords();
        if (records.isEmpty()) {
            return page;
        }

        // 查询对账单和转帐凭证
        List<Long> idList = records.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        Map<Long, List<FileInfoVo>> receiptFileMap = ObsFileUtil.getFileMap(idList, 1);
        Map<Long, List<FileInfoVo>> proofFileMap = ObsFileUtil.getFileMap(idList, 2);

        Map<Long, Map<String, Object>> map = orderService.listMaps(Wrappers.<OrderInfo>query()
                        .select(
                                "statement_of_account_id as statementOfAccountId",
                                "sum(total_amount) as amount",
                                "count(0) as orderNum"
                        )
                        .in("statement_of_account_id", idList)
                        .groupBy("statement_of_account_id")
                )
                .stream()
                .collect(Collectors.toMap(
                        item -> Convert.toLong(item.get("statementOfAccountId")),
                        Function.identity()
                ));

        // 获取售后订单
        Map<Long, OrderExchangeAccountBo> exchangeAccountBoMap = orderExchangeService.orderExchangeAccountStatistics(idList);

        for (StatementOfAccountVo record : records) {
            record.setReceiptFileList(receiptFileMap.getOrDefault(record.getId(), Collections.emptyList()));
            record.setProofFileList(proofFileMap.getOrDefault(record.getId(), Collections.emptyList()));
            Map<String, Object> tempMap = map.get(record.getId());
            if (tempMap == null) {
                record.setOrderNum(0);
                record.setAmount(BigDecimal.ZERO);
            } else {
                record.setOrderNum(Convert.toInt(tempMap.get("orderNum"), 0));
                record.setAmount(Convert.toBigDecimal(tempMap.get("amount"), BigDecimal.ZERO));
            }
            OrderExchangeAccountBo orderExchangeAccountBo = exchangeAccountBoMap.get(record.getId());
            if (orderExchangeAccountBo != null) {
                // 对账金额减去退货金额，加每个退货sku的两元质检费
                record.setOrderNum(record.getOrderNum() + orderExchangeAccountBo.getOrderNum());
                record.setAmount(record.getAmount()
                        .subtract(orderExchangeAccountBo.getReturnAmount())
                        .add(BigDecimal.valueOf(orderExchangeAccountBo.getOrderSkuNum()).multiply(BigDecimal.valueOf(2))));
            }
        }

        return page;
    }

    @Override
    public List<ReconciliationDetailVo> detail(Long id) {

        List<OrderInfo> orderInfoList = orderService.list(q -> q.eq(OrderInfo::getStatementOfAccountId, id));

        List<ReconciliationDetailVo> result = orderInfoList.stream().map(item -> ReconciliationDetailVo.builder()
                .orderId(item.getId())
                .departmentId(item.getDepartmentId())
                .orderCode(item.getCode())
                .totalAmount(item.getTotalAmount())
                .productTotalAmount(item.getProductTotalAmount())
                .customProcessingFee(item.getCustomProcessingFee())
                .lssueFee(item.getLssueFee())
                .deliveryMaterialsFee(item.getDeliveryMaterialsFee())
                .packingLabor(item.getPackingLabor())
                .packagingMaterialCost(item.getPackagingMaterialCost())
                .managementFee(item.getManagementFee())
                .build()
        ).collect(Collectors.toList());

        departmentService.attributeAssign(result, ReconciliationDetailVo::getDepartmentId, (item, department) -> {
            item.setDepartmentName(department.getName());
        });

        return result;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(StatementOfAccountDto statementOfAccountDto) {
        List<Long> orderIdList = statementOfAccountDto.getOrderIdList();

        // 判断当前事业部今日是否生成对账单
        Long departmentId = statementOfAccountDto.getDepartmentId();
        Date date = new Date();
        Date beginDate = DateUtil.beginOfDay(date);
        Date endDate = DateUtil.endOfDay(date);
        StatementOfAccount statementOfAccount = this.getOne(q -> q
                .eq(StatementOfAccount::getDepartmentId, departmentId)
                .between(StatementOfAccount::getTimePeriod, beginDate, endDate));

        if (statementOfAccount == null) {
            statementOfAccount = new StatementOfAccount();
            statementOfAccount.setDepartmentId(departmentId);
            statementOfAccount.setCode(CodeEnum.STATEMENT_OF_ACCOUNT_CODE.getCode());
            statementOfAccount.setType(2);
            statementOfAccount.setTimePeriod(new Date());
            this.save(statementOfAccount);
        }
        Long accountId = statementOfAccount.getId();

        if (ObjectUtil.isNotEmpty(orderIdList)) {
            orderService.update(q -> q
                    .in(BaseIdPo::getId, orderIdList)
                    .set(OrderInfo::getStatementOfAccountId, accountId)
            );
        }

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(StatementOfAccountDto statementOfAccountDto) {
        Long statementOfAccountId = statementOfAccountDto.getId();
        List<Long> orderIdList = statementOfAccountDto.getOrderIdList();

        Assert.notNull(statementOfAccountId, "对账单id不能为空");

        if (ObjectUtil.isNotEmpty(orderIdList)) {
            orderService.update(q -> q
                    .in(BaseIdPo::getId, orderIdList)
                    .set(OrderInfo::getStatementOfAccountId, statementOfAccountId)
            );
        }

        orderService.update(q -> q
                .eq(OrderInfo::getStatementOfAccountId, statementOfAccountId)
                .notIn(ObjectUtil.isNotEmpty(orderIdList), BaseIdPo::getId, orderIdList)
                .set(OrderInfo::getStatementOfAccountId, null)
        );

    }

    @DSTransactional
    @Override
    public void delete(Long id) {

        orderService.update(q -> q
                .eq(OrderInfo::getStatementOfAccountId, id)
                .set(OrderInfo::getStatementOfAccountId, null)
        );

        this.removeById(id);
        ObsFileUtil.removeFile(id);
    }

    @Override
    public void editFile(FileUploadDto dto, int type) {
        ObsFileUtil.editFile(dto.getFileList(), dto.getId(), type);
    }

    @Override
    public List<DocumentBySkuVo> getSkuDocument(List<Long> statementOfAccountIdList, Integer orderClassify) {
        // 获取售后sku记录
        if (Objects.equals(orderClassify, OrderClassifyEnum.AFTER_SALE_ORDER.getKey())) {
            return getAfterSaleSkuDocument(statementOfAccountIdList);
        }

        // 获取订单id列表
        List<Long> orderIdList = getOrderIdList(statementOfAccountIdList, orderClassify);

        if (orderIdList.isEmpty()) {
            return Collections.emptyList();
        }

        // 获取订单sku
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.in(OrderSku::getOrderId, orderIdList));

        // 生成结果集
        List<DocumentBySkuVo> documentBySkuVoList = orderSkuList.stream()
                .map(item -> DocumentBySkuVo.builder()
                        .skuSpecId(item.getSkuSpecId())
                        .orderId(item.getOrderId())
                        .quantity(item.getQuantity())
                        .unitPrice(item.getUnitPrice()
                                .add(item.getCustomProcessingFee())
                                .add(item.getLssueFee())
                                .add(item.getDeliveryMaterialsFee())
                                .add(item.getPackingLabor())
                                .add(item.getPackagingMaterialCost())
                                .add(item.getManagementFee())
                        )
                        .proofingFee(item.getProofingFee())
                        .build())
                .peek(item -> item.setSubtotal(item.getQuantity().multiply(item.getUnitPrice())
                        .add(item.getProofingFee())
                        .setScale(2, RoundingMode.HALF_UP)))
                .peek(item -> item.setTotal(item.getSubtotal()))
                .peek(item -> item.setUnitPrice(item.getUnitPrice()
                        .add(item.getProofingFee().divide(item.getQuantity(), 2, RoundingMode.HALF_UP))
                        .setScale(2, RoundingMode.HALF_UP)))
                .collect(Collectors.toList());

        // 查询赠品sku
        List<Long> skuSpecIds = documentBySkuVoList.stream().map(DocumentBySkuVo::getSkuSpecId).collect(Collectors.toList());
        Map<Long, SkuSpec> skuSpecMap = skuSpecService.byIdsToMap(skuSpecIds);
        Set<Long> giftSkuSpecIds = skuSpecMap.values().stream()
                .filter(item -> Objects.equals(item.getGiftTag(), StatusConstant.YES))
                .map(BaseIdPo::getId)
                .collect(Collectors.toSet());

        // 赋值sku规格品名和品号
        for (DocumentBySkuVo documentBySkuVo : documentBySkuVoList) {
            SkuSpec skuSpec = skuSpecMap.get(documentBySkuVo.getSkuSpecId());
            if (skuSpec == null) {
                continue;
            }
            documentBySkuVo.setSkuSpecCode(skuSpec.getCode());
            documentBySkuVo.setSkuSpecName(skuSpec.getName());
        }

        // 合并单价、小计、sku规格id系统的对账数据
        Collection<DocumentBySkuVo> documentBySkuVoCollection = documentBySkuVoList.stream()
                .filter(item -> giftSkuSpecIds.isEmpty() || !giftSkuSpecIds.contains(item.getSkuSpecId()))
                .sorted(comparing(DocumentBySkuVo::getSkuSpecCode))
                .collect(Collectors.toMap(
                        item -> item.getUnitPrice() + ":" + item.getSkuSpecId(),
                        Function.identity(),
                        (v1, v2) -> {
                            v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                            v1.setSubtotal(v1.getSubtotal().add(v2.getSubtotal()));
                            v1.setTotal(v1.getTotal().add(v2.getTotal()));
                            return v1;
                        }
                )).values();

        return new ArrayList<>(documentBySkuVoCollection);
    }

    @Override
    public List<DocumentByBomVo> getBomDocument(List<Long> statementOfAccountIdList, Integer orderClassify) {
        // 获取售后bom记录
        if (Objects.equals(orderClassify, OrderClassifyEnum.AFTER_SALE_ORDER.getKey())) {
            return getAfterSaleBomDocument(statementOfAccountIdList);
        }

        // 获取订单id列表
        List<Long> orderIdList = getOrderIdList(statementOfAccountIdList, orderClassify);
        if (orderIdList.isEmpty()) {
            return Collections.emptyList();
        }

        // 获取订单sku
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.in(OrderSku::getOrderId, orderIdList));

        // 筛选掉赠品sku
        List<Long> skuSpecIds = orderSkuList.stream().map(OrderSku::getSkuSpecId).collect(Collectors.toList());
        Set<Long> giftSkuSpecIds = skuSpecService.getGiftIdListByIdList(skuSpecIds);

        // 主材bom
        List<DocumentByBomVo> result = orderSkuList.stream()
                .filter(item -> giftSkuSpecIds.isEmpty() || !giftSkuSpecIds.contains(item.getSkuSpecId()))
                .map(item -> DocumentByBomVo.builder()
                        .bomSpecId(item.getBomSpecId())
                        .orderId(item.getOrderId())
                        .quantity(item.getQuantity())
                        .unitPrice(item.getUnitPrice())
                        .laserLogoSummary((Objects.equals(item.getCustomProcessingType(), "20")
                                ? item.getCustomProcessingFee().multiply(item.getQuantity()) : BigDecimal.ZERO))
                        .laserMitochondrialSummary((Objects.equals(item.getCustomProcessingType(), "10")
                                ? item.getCustomProcessingFee().multiply(item.getQuantity()) : BigDecimal.ZERO))
                        .lssueFeeSummary(item.getLssueFee().multiply(item.getQuantity()))
                        .deliveryMaterialsFeeSummary(item.getDeliveryMaterialsFee().multiply(item.getQuantity()).setScale(2, RoundingMode.HALF_UP))
                        .packingLaborSummary(item.getPackingLabor().multiply(item.getQuantity()))
                        .managementFeeSummary(item.getManagementFee().multiply(item.getQuantity()))
                        .proofingFeeSummary(item.getProofingFee())
                        .build())
                .peek(item -> item.setSubtotal(
                        item.getUnitPrice()
                                .multiply(item.getQuantity())
                                .add(item.getLaserLogoSummary())
                                .add(item.getLaserMitochondrialSummary())
                                .add(item.getLssueFeeSummary())
                                .add(item.getDeliveryMaterialsFeeSummary())
                                .add(item.getPackingLaborSummary())
                                .add(item.getManagementFeeSummary())
                                .add(item.getProofingFeeSummary())
                                .setScale(2, RoundingMode.HALF_UP)
                ))
                .peek(item -> item.setTotal(item.getSubtotal()))
                .collect(Collectors.toList());

        // 赋值主材bom品名品号
        bomSpecService.attributeAssign(result, DocumentByBomVo::getBomSpecId, (item, bomSpec) -> {
            item.setBomSpecCode(bomSpec.getCode());
            item.setBomSpecName(bomSpec.getName());
        });

        // 合并同加个同规格主材bom，数量相加
        Collection<DocumentByBomVo> documentByBomVoCollection = result.stream()
                .sorted(comparing(DocumentByBomVo::getBomSpecCode))
                .collect(Collectors.toMap(
                        item -> item.getUnitPrice() + ":" + item.getBomSpecId(),
                        Function.identity(),
                        (v1, v2) -> {
                            v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                            v1.setLaserLogoSummary(v1.getLaserLogoSummary().add(v2.getLaserLogoSummary()));
                            v1.setLaserMitochondrialSummary(v1.getLaserMitochondrialSummary().add(v2.getLaserMitochondrialSummary()));
                            v1.setLssueFeeSummary(v1.getLssueFeeSummary().add(v2.getLssueFeeSummary()));
                            v1.setDeliveryMaterialsFeeSummary(v1.getDeliveryMaterialsFeeSummary().add(v2.getDeliveryMaterialsFeeSummary()));
                            v1.setPackingLaborSummary(v1.getPackingLaborSummary().add(v2.getPackingLaborSummary()));
                            v1.setManagementFeeSummary(v1.getManagementFeeSummary().add(v2.getManagementFeeSummary()));
                            v1.setProofingFeeSummary(v1.getProofingFeeSummary().add(v2.getProofingFeeSummary()));
                            v1.setSubtotal(v1.getSubtotal().add(v2.getSubtotal()));
                            v1.setTotal(v1.getSubtotal());
                            return v1;
                        }
                )).values();

        // 赋值合并后结果
        result = new ArrayList<>(documentByBomVoCollection);
        // 按品名升序排序
        result.sort(comparing(DocumentByBomVo::getBomSpecName));

        Map<Long, BigDecimal> orderSkuMap = orderSkuList.stream().collect(Collectors.toMap(BaseIdPo::getId, OrderSku::getQuantity));

        // 包材bom
        List<OrderSkuBom> orderSkuBomList = orderSkuBomService.list(q -> q.in(OrderSkuBom::getOrderId, orderIdList));
        List<DocumentByBomVo> bomVoList = orderSkuBomList.stream().map(item ->
                DocumentByBomVo.builder()
                        .bomSpecId(item.getBomSpecId())
                        .quantity(item.getQuantity().multiply(orderSkuMap.get(item.getOrderSkuId())))
                        .unitPrice(item.getUnitPrice())
                        .subtotal(item.getUnitPrice().multiply(item.getQuantity().multiply(orderSkuMap.get(item.getOrderSkuId()))))
                        .total(item.getUnitPrice().multiply(item.getQuantity().multiply(orderSkuMap.get(item.getOrderSkuId()))))
                        .build()
        ).collect(Collectors.toList());

        // 赋值包材bom品名品号
        bomSpecService.attributeAssign(bomVoList, DocumentByBomVo::getBomSpecId, (item, bomSpec) -> {
            item.setBomSpecCode(bomSpec.getCode());
            item.setBomSpecName(bomSpec.getName());
        });

        // 合并同加个同规格包材bom，数量相加
        Collection<DocumentByBomVo> bomValues = bomVoList.stream()
                .sorted(comparing(DocumentByBomVo::getBomSpecCode))
                .collect(Collectors.toMap(
                        item -> item.getUnitPrice() + ":" + item.getBomSpecId(),
                        Function.identity(),
                        (v1, v2) -> {
                            v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                            v1.setSubtotal(v1.getSubtotal().add(v2.getSubtotal()));
                            v1.setTotal(v1.getSubtotal());
                            return v1;
                        }
                )).values();

        // 合并主材包材
        result.addAll(bomValues);

        return result;
    }

    @Override
    public List<DocumentByOrderVo> getOrderDocument(List<Long> statementOfAccountId, Integer orderClassify) {

        Assert.notEmpty(statementOfAccountId, "对账单id不能为空");

        // 获取售后bom记录
        if (Objects.equals(orderClassify, OrderClassifyEnum.AFTER_SALE_ORDER.getKey())) {
            return getAfterSaleOrderDocument(statementOfAccountId);
        }

        List<OrderInfo> orderList = orderService.list(q -> q
                .in(OrderInfo::getStatementOfAccountId, statementOfAccountId)
                .eq(OrderInfo::getClassify, orderClassify));

        return getOrderDocumentByOrderList(orderList);
    }

    @Override
    public List<DocumentByOrderVo> getOrderDocumentByOrderList(List<OrderInfo> orderList) {
        if (orderList.isEmpty()) {
            return Collections.emptyList();
        }

        List<DocumentByOrderVo> result = orderList.stream().map(item -> {
            DocumentByOrderVo documentByOrderVo = new DocumentByOrderVo();
            documentByOrderVo.setOrderId(item.getId());
            documentByOrderVo.setWlnCreateTime(ObjectUtil.defaultIfNull(item.getWlnCreateTime(), item.getCreateTime()));
            documentByOrderVo.setCode(item.getCode());
            documentByOrderVo.setWlnCode(item.getWlnCode());
            documentByOrderVo.setTotal(item.getTotalAmount());

            documentByOrderVo.setSourcePlatform(item.getSourcePlatform());
            documentByOrderVo.setShopName(item.getShopName());
            documentByOrderVo.setDepartmentId(item.getDepartmentId());
            documentByOrderVo.setStatementOfAccountId(item.getStatementOfAccountId());

            return documentByOrderVo;
        }).collect(Collectors.toList());

        List<Long> orderIdList = orderList.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.in(OrderSku::getOrderId, orderIdList));
        // 筛选掉赠品sku
        List<Long> skuSpecIds = orderSkuList.stream().map(OrderSku::getSkuSpecId).collect(Collectors.toList());
        Set<Long> giftSkuSpecIds = skuSpecService.getGiftIdListByIdList(skuSpecIds);
        List<OrderSku> tempOrderSkuList = orderSkuList.stream()
                .filter(item -> giftSkuSpecIds.isEmpty() || !giftSkuSpecIds.contains(item.getSkuSpecId()))
                .collect(Collectors.toList());

        SecurityContext context = SecurityContextHolder.getContext();

        CompletableFuture<List<DocumentByOrderVo.SkuSpec>> skuSpecListCompletableFuture = CompletableFuture.supplyAsync(
                () -> {
                    SecurityContextHolder.setContext(context);
                    return getSkuSpecList(tempOrderSkuList);
                }, threadPoolExecutor);

        CompletableFuture<List<DocumentByOrderVo.BomSpec>> bomSpecListCompletableFuture = CompletableFuture.supplyAsync(
                () -> {
                    SecurityContextHolder.setContext(context);
                    return getBomSpecList(orderIdList, tempOrderSkuList);
                }, threadPoolExecutor);

        // 获取订单bom
        List<DocumentByOrderVo.BomSpec> bomSpecList = bomSpecListCompletableFuture.join();

        // sku赋值bom
        Map<Long, List<DocumentByOrderVo.BomSpec>> bomSpecMap = bomSpecList.stream()
                .collect(Collectors.groupingBy(DocumentByOrderVo.BomSpec::getOrderSkuId));

        // 获取订单sku
        List<DocumentByOrderVo.SkuSpec> skuSpecList = skuSpecListCompletableFuture.join();

        for (DocumentByOrderVo.SkuSpec skuSpec : skuSpecList) {
            skuSpec.setBomSpecList(bomSpecMap.getOrDefault(skuSpec.getOrderSkuId(), Collections.emptyList()));
        }

        // 订单赋值sku
        Map<Long, List<DocumentByOrderVo.SkuSpec>> skuSpecMap = skuSpecList.stream()
                .collect(Collectors.groupingBy(DocumentByOrderVo.SkuSpec::getOrderId));
        result.forEach(item -> item.setSkuSpecList(skuSpecMap.getOrDefault(item.getOrderId(), Collections.emptyList())));

        return result;
    }

    @Override
    public void export(Long id, Integer orderClassify, int type) {

        StatementOfAccount statementOfAccount = getById(id);
        Assert.notNull(statementOfAccount, "没有找到对账单");

        Department department = departmentService.getById(statementOfAccount.getDepartmentId());
        Assert.notNull(department, "没有找到事业部");

        Date timePeriod = statementOfAccount.getTimePeriod();
        String date = timePeriod != null ? DateUtil.formatDate(timePeriod) : StringPool.EMPTY;
        String departmentName = department.getName();

        List<Long> idList = Collections.singletonList(id);

        switch (type) {
            case 1:
                statementOfAccountExportService.exportDocumentBySku(idList, departmentName, date, date, orderClassify);
                break;
            case 2:
                statementOfAccountExportService.exportDocumentByBom(idList, departmentName, date, date, orderClassify);
                break;
            case 3:
                excelGenerateLogService.generateExcel(ExcelTypeEnum.STATISTICS_DOCUMENT_BY_ORDER,
                        new DocumentByOrderExcelExportStrategy(idList, departmentName, date, date, orderClassify),
                        DateUtil.formatDate(new Date()) + " " + OrderClassifyEnum.getEnum(orderClassify).getValue() + "-订单对账单");
                break;
            default:
                throw new ServiceException("未知对账单类型");
        }

    }

    @Override
    public String getOrderWlnCodeStr(List<Long> idList) {
        List<OrderInfo> list = orderService.list(q -> q.in(OrderInfo::getStatementOfAccountId, idList));
        List<String> wlnCodeList = list.stream().map(OrderInfo::getWlnCode).filter(ObjectUtil::isNotNull).collect(Collectors.toList());
        return JSON.toJSONString(wlnCodeList);
    }

    @Override
    public StatementOrderClassifyTotalCountVo getOrderClassifyTotalCount(List<Long> idList) {
        Assert.notEmpty(idList, "对账单id不能为空");
        StatementOrderClassifyTotalCountVo vo = orderService.getOrderClassifyTotalCount(idList);
        if (vo == null) {
            vo = new StatementOrderClassifyTotalCountVo();
            vo.setWlnOrderCount(0);
            vo.setPurchaseOrderCount(0);
            vo.setOutsourceOrderCount(0);
            vo.setAfterSaleOrderCount(0);
            vo.setNoReasonOrderCount(0);
        }
        // 查询售后订单数量
        long count = orderExchangeService.count(q -> q
                .eq(OrderExchange::getStatus, StatusConstant.YES)
                .in(OrderExchange::getStatementOfAccountId, idList));
        vo.setAfterSaleOrderCount((int) count);
        return vo;
    }

    /**
     * 根据对账单id获取对账订单id列表
     */
    private List<Long> getOrderIdList(List<Long> statementOfAccountIdList, Integer orderClassify) {
        Assert.notEmpty(statementOfAccountIdList, "对账单id不能为空");
        List<OrderInfo> orderList = orderService.list(q -> q.in(OrderInfo::getStatementOfAccountId, statementOfAccountIdList)
                .eq(OrderInfo::getClassify, orderClassify));
        return orderList.stream().map(BaseIdPo::getId).collect(Collectors.toList());
    }

    /**
     * 获取订单sku
     */
    private List<DocumentByOrderVo.SkuSpec> getSkuSpecList(List<OrderSku> orderSkuList) {

        List<DocumentByOrderVo.SkuSpec> skuSpecList = orderSkuList.stream()
                .map(item -> {
                    DocumentByOrderVo.SkuSpec skuSpec = new DocumentByOrderVo.SkuSpec();
                    skuSpec.setOrderId(item.getOrderId());
                    skuSpec.setOrderSkuId(item.getId());
                    skuSpec.setSkuSpecId(item.getSkuSpecId());
                    skuSpec.setQuantity(item.getQuantity());
                    skuSpec.setProofingFee(item.getProofingFee());
                    skuSpec.setUnitPrice(item.getUnitPrice()
                            .add(item.getCustomProcessingFee())
                            .add(item.getLssueFee())
                            .add(item.getDeliveryMaterialsFee())
                            .add(item.getPackingLabor())
                            .add(item.getPackagingMaterialCost())
                            .add(item.getManagementFee()));
                    return skuSpec;
                })
                .peek(item -> item.setSubtotal(item.getQuantity().multiply(item.getUnitPrice())
                        .add(item.getProofingFee())
                        .setScale(2, RoundingMode.HALF_UP)))
                .peek(item -> item.setUnitPrice(item.getUnitPrice()
                        .add(item.getProofingFee().divide(item.getQuantity(), 2, RoundingMode.HALF_UP))
                        .setScale(2, RoundingMode.HALF_UP)))
                .collect(Collectors.toList());

        skuSpecService.attributeAssign(skuSpecList, DocumentByOrderVo.SkuSpec::getSkuSpecId, (item, skuSpec) -> {
            item.setSkuSpecCode(skuSpec.getCode());
            item.setSkuSpecName(skuSpec.getName());
        });

        return skuSpecList;
    }

    /**
     * 获取订单bom
     */
    private List<DocumentByOrderVo.BomSpec> getBomSpecList(List<Long> orderIdList, List<OrderSku> orderSkuList) {

        // 主材
        List<DocumentByOrderVo.BomSpec> bomSpecList = orderSkuList.stream()
                .map(item -> {
                    DocumentByOrderVo.BomSpec bomSpec = new DocumentByOrderVo.BomSpec();
                    bomSpec.setOrderSkuId(item.getId());
                    bomSpec.setBomSpecId(item.getBomSpecId());
                    bomSpec.setQuantity(item.getQuantity());
                    bomSpec.setUnitPrice(item.getUnitPrice());
                    bomSpec.setCustomProcessingType(item.getCustomProcessingType());
                    bomSpec.setLaserLogoSummary((Objects.equals(item.getCustomProcessingType(), "20")
                            ? item.getCustomProcessingFee() : BigDecimal.ZERO));
                    bomSpec.setLaserMitochondrialSummary((Objects.equals(item.getCustomProcessingType(), "10")
                            ? item.getCustomProcessingFee() : BigDecimal.ZERO));
                    bomSpec.setLssueFeeSummary(item.getLssueFee());
                    bomSpec.setDeliveryMaterialsFeeSummary(item.getDeliveryMaterialsFee());
                    bomSpec.setPackingLaborSummary(item.getPackingLabor());
                    bomSpec.setManagementFeeSummary(item.getManagementFee());
                    bomSpec.setProofingFeeSummary(item.getProofingFee().divide(item.getQuantity(), 2, RoundingMode.HALF_UP));
                    return bomSpec;
                })
                .collect(Collectors.toList());

        Map<Long, BigDecimal> map = bomSpecList.stream().collect(
                Collectors.toMap(DocumentByOrderVo.BomSpec::getOrderSkuId, DocumentByOrderVo.BomSpec::getQuantity));

        // 包材
        List<OrderSkuBom> orderSkuBomList = orderSkuBomService.list(q -> q.in(OrderSkuBom::getOrderId, orderIdList));
        List<DocumentByOrderVo.BomSpec> packBomSpecList = orderSkuBomList.stream()
                .map(item -> {
                    DocumentByOrderVo.BomSpec bomSpec = new DocumentByOrderVo.BomSpec();
                    bomSpec.setOrderSkuId(item.getOrderSkuId());
                    bomSpec.setBomSpecId(item.getBomSpecId());
                    bomSpec.setQuantity(item.getQuantity());
                    bomSpec.setUnitPrice(item.getUnitPrice());
                    return bomSpec;
                })
                .peek(item -> item.setQuantity(ObjectUtil.equals(map.get(item.getOrderSkuId()), BigDecimal.ZERO) ? item.getQuantity() : map.get(item.getOrderSkuId()).multiply(item.getQuantity())))
                .collect(Collectors.toList());

        // 赋值
        bomSpecList.addAll(packBomSpecList);

        List<Long> bomSpecIdList = bomSpecList.stream().map(DocumentByOrderVo.BomSpec::getBomSpecId).collect(Collectors.toList());
        Map<Long, BomSpecBo> bomSpecBoMap = skuSpecService.getBomSpecBoByIdList(bomSpecIdList);

        for (DocumentByOrderVo.BomSpec bomSpec : bomSpecList) {
            BomSpecBo bomSpecBo = bomSpecBoMap.get(bomSpec.getBomSpecId());

            bomSpec.setBomSpecCode(bomSpecBo.getBomSpecCode());
            bomSpec.setBomSpecName(bomSpecBo.getBomSpecName());
            bomSpec.setClassifyName(bomSpecBo.getClassifyName());
            bomSpec.setUnit(bomSpecBo.getUnit());
        }

        return bomSpecList;
    }

    /**
     * 获取售后订单bom
     */
    private List<DocumentByOrderVo.BomSpec> getAfterSaleBomSpecList(List<OrderSku> orderSkuList, List<OrderSkuBom> orderSkuBomList) {
        // 主材
        List<DocumentByOrderVo.BomSpec> bomSpecList = orderSkuList.stream()
                .map(item -> {
                    DocumentByOrderVo.BomSpec bomSpec = new DocumentByOrderVo.BomSpec();
                    bomSpec.setOrderSkuId(item.getId());
                    bomSpec.setBomSpecId(item.getBomSpecId());
                    bomSpec.setQuantity(item.getQuantity());
                    bomSpec.setUnitPrice(item.getUnitPrice());
                    bomSpec.setCustomProcessingType(item.getCustomProcessingType());
                    bomSpec.setLaserLogoSummary((Objects.equals(item.getCustomProcessingType(), "20")
                            ? item.getCustomProcessingFee() : BigDecimal.ZERO));
                    bomSpec.setLaserMitochondrialSummary((Objects.equals(item.getCustomProcessingType(), "10")
                            ? item.getCustomProcessingFee() : BigDecimal.ZERO));
                    bomSpec.setLssueFeeSummary(item.getLssueFee());
                    bomSpec.setDeliveryMaterialsFeeSummary(item.getDeliveryMaterialsFee());
                    bomSpec.setPackingLaborSummary(item.getPackingLabor());
                    bomSpec.setManagementFeeSummary(item.getManagementFee());
                    bomSpec.setProofingFeeSummary(item.getProofingFee().divide(item.getQuantity(), 2, RoundingMode.HALF_UP));
                    return bomSpec;
                })
                .collect(Collectors.toList());

        Map<Long, BigDecimal> map = bomSpecList.stream().collect(
                Collectors.toMap(DocumentByOrderVo.BomSpec::getOrderSkuId, DocumentByOrderVo.BomSpec::getQuantity));

        // 包材
        List<DocumentByOrderVo.BomSpec> packBomSpecList = orderSkuBomList.stream()
                .map(item -> {
                    DocumentByOrderVo.BomSpec bomSpec = new DocumentByOrderVo.BomSpec();
                    bomSpec.setOrderSkuId(item.getOrderSkuId());
                    bomSpec.setBomSpecId(item.getBomSpecId());
                    bomSpec.setQuantity(item.getQuantity());
                    bomSpec.setUnitPrice(item.getUnitPrice());
                    return bomSpec;
                })
                .peek(item -> item.setQuantity(ObjectUtil.equals(map.get(item.getOrderSkuId()), BigDecimal.ZERO) ? item.getQuantity() : map.get(item.getOrderSkuId()).multiply(item.getQuantity())))
                .collect(Collectors.toList());

        // 赋值
        bomSpecList.addAll(packBomSpecList);

        List<Long> bomSpecIdList = bomSpecList.stream().map(DocumentByOrderVo.BomSpec::getBomSpecId).collect(Collectors.toList());
        Map<Long, BomSpecBo> bomSpecBoMap = skuSpecService.getBomSpecBoByIdList(bomSpecIdList);

        for (DocumentByOrderVo.BomSpec bomSpec : bomSpecList) {
            BomSpecBo bomSpecBo = bomSpecBoMap.get(bomSpec.getBomSpecId());

            bomSpec.setBomSpecCode(bomSpecBo.getBomSpecCode());
            bomSpec.setBomSpecName(bomSpecBo.getBomSpecName());
            bomSpec.setClassifyName(bomSpecBo.getClassifyName());
            bomSpec.setUnit(bomSpecBo.getUnit());
        }

        return bomSpecList;
    }

    /**
     * 获取售后订单记录
     * @param statementOfAccountId
     * @return
     */
    private List<DocumentByOrderVo> getAfterSaleOrderDocument(List<Long> statementOfAccountId) {
        // 获取订单id列表
        List<OrderExchange> list = orderExchangeService.list(q -> q
                        .eq(OrderExchange::getStatus, StatusConstant.YES)
                        .in(OrderExchange::getStatementOfAccountId, statementOfAccountId)
                        .select(BaseIdPo::getId, OrderExchange::getOrderInfoId));
        if (list.isEmpty()) {
            return Collections.emptyList();
        }
        List<Long> orderExchangeIdList = list.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        List<Long> orderIdList = list.stream().map(OrderExchange::getOrderInfoId).collect(Collectors.toList());

        List<OrderExchangeDetail> exchangeDetailList = orderExchangeDetailService.list(q -> q
                .in(OrderExchangeDetail::getOrderExchangeId, orderExchangeIdList));

        if (exchangeDetailList.isEmpty()) {
            return Collections.emptyList();
        }
        Map<Long, BigDecimal> exchangeDetailMap = exchangeDetailList.stream().collect(Collectors.toMap(
                OrderExchangeDetail::getOrderSkuId,
                OrderExchangeDetail::getQuantity,
                BigDecimal::add));

        // 获取订单
        List<OrderInfo> orderList = orderService.list(q -> q.in(BaseIdPo::getId, orderIdList));
        // 获取订单sku
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.in(BaseIdPo::getId, exchangeDetailMap.keySet()));

        List<OrderSkuBom> orderSkuBomList = orderSkuBomService.list(q -> q.in(OrderSkuBom::getOrderSkuId, exchangeDetailMap.keySet()));
        // 筛选出可二次利用的包材
        List<OrderSkuBom> tempOrderSkuBomList = this.getReusableOrderSkuBomList(orderSkuBomList);
        Map<Long, BigDecimal> packagingMaterialCostMap = tempOrderSkuBomList.stream()
                .peek(item -> item.setUnitPrice(item.getUnitPrice().negate()))
                .collect(Collectors.toMap(
                        OrderSkuBom::getOrderSkuId,
                        v -> v.getUnitPrice().multiply(v.getQuantity()),
                        BigDecimal::add
                ));

        // 将订单sku数量更新为退货数量，退货只退裸垫单价和可二次利用的包材费
        for (OrderSku orderSku : orderSkuList) {
            orderSku.setQuantity(exchangeDetailMap.get(orderSku.getId()));
            orderSku.setProofingFee(BigDecimal.ZERO);
            orderSku.setLssueFee(BigDecimal.ZERO);
            orderSku.setPackingLabor(BigDecimal.ZERO);
            orderSku.setCustomProcessingFee(BigDecimal.ZERO);
            orderSku.setUnitPrice(orderSku.getUnitPrice().negate());
            orderSku.setDeliveryMaterialsFee(BigDecimal.ZERO);
            orderSku.setManagementFee(BigDecimal.ZERO);
            orderSku.setPackagingMaterialCost(packagingMaterialCostMap.getOrDefault(orderSku.getId(), BigDecimal.ZERO));
        }

        List<DocumentByOrderVo> result = orderList.stream().map(item -> {
            DocumentByOrderVo documentByOrderVo = new DocumentByOrderVo();
            documentByOrderVo.setOrderId(item.getId());
            documentByOrderVo.setWlnCreateTime(ObjectUtil.defaultIfNull(item.getWlnCreateTime(), item.getCreateTime()));
            documentByOrderVo.setCode(item.getCode());
            documentByOrderVo.setWlnCode(item.getWlnCode());

            documentByOrderVo.setSourcePlatform(item.getSourcePlatform());
            documentByOrderVo.setShopName(item.getShopName());
            documentByOrderVo.setDepartmentId(item.getDepartmentId());

            return documentByOrderVo;
        }).collect(Collectors.toList());

        SecurityContext context = SecurityContextHolder.getContext();

        CompletableFuture<List<DocumentByOrderVo.SkuSpec>> skuSpecListCompletableFuture = CompletableFuture.supplyAsync(
                () -> {
                    SecurityContextHolder.setContext(context);
                    return getSkuSpecList(orderSkuList);
                }, threadPoolExecutor);

        CompletableFuture<List<DocumentByOrderVo.BomSpec>> bomSpecListCompletableFuture = CompletableFuture.supplyAsync(
                () -> {
                    SecurityContextHolder.setContext(context);
                    return getAfterSaleBomSpecList(orderSkuList, tempOrderSkuBomList);
                }, threadPoolExecutor);

        // 获取订单bom
        List<DocumentByOrderVo.BomSpec> bomSpecList = bomSpecListCompletableFuture.join();

        // sku赋值bom
        Map<Long, List<DocumentByOrderVo.BomSpec>> bomSpecMap = bomSpecList.stream()
                .collect(Collectors.groupingBy(DocumentByOrderVo.BomSpec::getOrderSkuId));

        // 获取订单sku
        List<DocumentByOrderVo.SkuSpec> skuSpecList = skuSpecListCompletableFuture.join();

        for (DocumentByOrderVo.SkuSpec skuSpec : skuSpecList) {
            skuSpec.setBomSpecList(bomSpecMap.getOrDefault(skuSpec.getOrderSkuId(), Collections.emptyList()));
            // 保存质检费,sku小计增加质检费
            skuSpec.setCheckFee(BigDecimal.valueOf(2).multiply(skuSpec.getQuantity()));
            skuSpec.setSubtotal(skuSpec.getSubtotal()
                    .add(skuSpec.getCheckFee()));
        }

        // 订单赋值sku
        Map<Long, List<DocumentByOrderVo.SkuSpec>> skuSpecMap = skuSpecList.stream()
                .collect(Collectors.groupingBy(DocumentByOrderVo.SkuSpec::getOrderId));
        result.forEach(item -> {
            List<DocumentByOrderVo.SkuSpec> specList = skuSpecMap.getOrDefault(item.getOrderId(), Collections.emptyList());
            item.setSkuSpecList(specList);
            BigDecimal total = specList.stream().map(DocumentByOrderVo.SkuSpec::getSubtotal).reduce(BigDecimal.ZERO, BigDecimal::add);
            item.setTotal(total);
        });

        return result;
    }

    /**
     * 获取售后bom记录
     * @param statementOfAccountIdList
     * @return
     */
    private List<DocumentByBomVo> getAfterSaleBomDocument(List<Long> statementOfAccountIdList) {
        Assert.notEmpty(statementOfAccountIdList, "对账单id不能为空");
        // 获取订单id列表
        List<Long> orderExchangeIdList = orderExchangeService.list(q -> q
                        .eq(OrderExchange::getStatus, StatusConstant.YES)
                        .in(OrderExchange::getStatementOfAccountId, statementOfAccountIdList)
                        .select(BaseIdPo::getId))
                .stream().map(BaseIdPo::getId).collect(Collectors.toList());

        if (orderExchangeIdList.isEmpty()) {
            return Collections.emptyList();
        }

        List<OrderExchangeDetail> exchangeDetailList = orderExchangeDetailService.list(q -> q
                .in(OrderExchangeDetail::getOrderExchangeId, orderExchangeIdList));

        if (exchangeDetailList.isEmpty()) {
            return Collections.emptyList();
        }
        Map<Long, BigDecimal> exchangeDetailMap = exchangeDetailList.stream().collect(Collectors.toMap(
                OrderExchangeDetail::getOrderSkuId,
                OrderExchangeDetail::getQuantity,
                BigDecimal::add));

        // 获取订单sku
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.in(BaseIdPo::getId, exchangeDetailMap.keySet()));

        // 主材bom
        List<DocumentByBomVo> result = orderSkuList.stream()
                .map(item -> DocumentByBomVo.builder()
                        .bomSpecId(item.getBomSpecId())
                        .orderId(item.getOrderId())
                        .quantity(exchangeDetailMap.get(item.getId()))
                        .unitPrice(item.getUnitPrice().negate())
                        .laserLogoSummary(BigDecimal.ZERO)
                        .laserMitochondrialSummary(BigDecimal.ZERO)
                        .lssueFeeSummary(BigDecimal.ZERO)
                        .deliveryMaterialsFeeSummary(BigDecimal.ZERO)
                        .packingLaborSummary(BigDecimal.ZERO)
                        .managementFeeSummary(BigDecimal.ZERO)
                        .checkFeeSummary(BigDecimal.valueOf(2).multiply(exchangeDetailMap.get(item.getId())))
                        .build())
                .peek(item -> item.setSubtotal(
                        item.getUnitPrice()
                                .multiply(item.getQuantity())
                                .add(item.getLaserLogoSummary())
                                .add(item.getLaserMitochondrialSummary())
                                .add(item.getLssueFeeSummary())
                                .add(item.getDeliveryMaterialsFeeSummary())
                                .add(item.getPackingLaborSummary())
                                .add(item.getManagementFeeSummary())
                                .add(item.getCheckFeeSummary())
                                .setScale(2, RoundingMode.HALF_UP)
                ))
                .peek(item -> item.setTotal(item.getSubtotal()))
                .collect(Collectors.toList());

        // 赋值主材bom品名品号
        bomSpecService.attributeAssign(result, DocumentByBomVo::getBomSpecId, (item, bomSpec) -> {
            item.setBomSpecCode(bomSpec.getCode());
            item.setBomSpecName(bomSpec.getName());
        });

        // 合并同加个同规格主材bom，数量相加
        Collection<DocumentByBomVo> documentByBomVoCollection = result.stream()
                .sorted(comparing(DocumentByBomVo::getBomSpecCode))
                .collect(Collectors.toMap(
                        item -> item.getUnitPrice() + ":" + item.getBomSpecId(),
                        Function.identity(),
                        (v1, v2) -> {
                            v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                            v1.setLaserLogoSummary(v1.getLaserLogoSummary().add(v2.getLaserLogoSummary()));
                            v1.setLaserMitochondrialSummary(v1.getLaserMitochondrialSummary().add(v2.getLaserMitochondrialSummary()));
                            v1.setLssueFeeSummary(v1.getLssueFeeSummary().add(v2.getLssueFeeSummary()));
                            v1.setDeliveryMaterialsFeeSummary(v1.getDeliveryMaterialsFeeSummary().add(v2.getDeliveryMaterialsFeeSummary()));
                            v1.setPackingLaborSummary(v1.getPackingLaborSummary().add(v2.getPackingLaborSummary()));
                            v1.setManagementFeeSummary(v1.getManagementFeeSummary().add(v2.getManagementFeeSummary()));
                            v1.setProofingFeeSummary(v1.getProofingFeeSummary().add(v2.getProofingFeeSummary()));
                            v1.setSubtotal(v1.getSubtotal().add(v2.getSubtotal()));
                            v1.setTotal(v1.getSubtotal());
                            return v1;
                        }
                )).values();

        // 赋值合并后结果
        result = new ArrayList<>(documentByBomVoCollection);
        // 按品名升序排序
        result.sort(comparing(DocumentByBomVo::getBomSpecName));

        Map<Long, BigDecimal> orderSkuMap = orderSkuList.stream().collect(Collectors.toMap(BaseIdPo::getId, OrderSku::getQuantity));

        // 包材bom
        List<OrderSkuBom> orderSkuBomList = orderSkuBomService.list(q -> q.in(OrderSkuBom::getOrderSkuId, exchangeDetailMap.keySet()));
        // 筛选出可二次利用的包材
        List<OrderSkuBom> tempOrderSkuBomList = this.getReusableOrderSkuBomList(orderSkuBomList);

        List<DocumentByBomVo> bomVoList = tempOrderSkuBomList.stream()
                .map(item ->
                        DocumentByBomVo.builder()
                                .bomSpecId(item.getBomSpecId())
                                .quantity(item.getQuantity().multiply(orderSkuMap.get(item.getOrderSkuId())))
                                .unitPrice(item.getUnitPrice().negate())
                                .subtotal(item.getUnitPrice().multiply(item.getQuantity().multiply(orderSkuMap.get(item.getOrderSkuId()))).negate())
                                .total(item.getUnitPrice().multiply(item.getQuantity().multiply(orderSkuMap.get(item.getOrderSkuId()))).negate())
                                .build()
                ).collect(Collectors.toList());

        // 赋值包材bom品名品号
        bomSpecService.attributeAssign(bomVoList, DocumentByBomVo::getBomSpecId, (item, bomSpec) -> {
            item.setBomSpecCode(bomSpec.getCode());
            item.setBomSpecName(bomSpec.getName());
        });

        // 合并同加个同规格包材bom，数量相加
        Collection<DocumentByBomVo> bomValues = bomVoList.stream()
                .sorted(comparing(DocumentByBomVo::getBomSpecCode))
                .collect(Collectors.toMap(
                        item -> item.getUnitPrice() + ":" + item.getBomSpecId(),
                        Function.identity(),
                        (v1, v2) -> {
                            v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                            v1.setSubtotal(v1.getSubtotal().add(v2.getSubtotal()));
                            v1.setTotal(v1.getSubtotal());
                            return v1;
                        }
                )).values();

        // 合并主材包材
        result.addAll(bomValues);

        return result;
    }

    /**
     * 获取售后sku记录
     * @param statementOfAccountIdList
     * @return
     */
    private List<DocumentBySkuVo> getAfterSaleSkuDocument(List<Long> statementOfAccountIdList) {
        Assert.notEmpty(statementOfAccountIdList, "对账单id不能为空");
        // 获取订单id列表
        List<Long> orderExchangeIdList = orderExchangeService.list(q -> q
                        .eq(OrderExchange::getStatus, StatusConstant.YES)
                        .in(OrderExchange::getStatementOfAccountId, statementOfAccountIdList)
                        .select(BaseIdPo::getId))
                .stream().map(BaseIdPo::getId).collect(Collectors.toList());

        if (orderExchangeIdList.isEmpty()) {
            return Collections.emptyList();
        }

        List<OrderExchangeDetail> exchangeDetailList = orderExchangeDetailService.list(q -> q
                .in(OrderExchangeDetail::getOrderExchangeId, orderExchangeIdList));
        Map<Long, BigDecimal> exchangeDetailMap = exchangeDetailList.stream().collect(Collectors.toMap(
                OrderExchangeDetail::getOrderSkuId,
                OrderExchangeDetail::getQuantity,
                BigDecimal::add));
        if (exchangeDetailList.isEmpty()) {
            return Collections.emptyList();
        }

        // 获取订单sku
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.in(BaseIdPo::getId, exchangeDetailMap.keySet()));
        List<Long> orderSkuIds = orderSkuList.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        // 获取可二次利用的包材金额
        List<OrderSkuBom> orderSkuBomList = orderSkuBomService.list(q -> q.in(OrderSkuBom::getOrderSkuId, orderSkuIds));
        List<OrderSkuBom> tempOrderSkuBomList = this.getReusableOrderSkuBomList(orderSkuBomList);
        Map<Long, BigDecimal> packagingMaterialCostMap = tempOrderSkuBomList.stream()
                .collect(Collectors.toMap(OrderSkuBom::getOrderSkuId, v -> v.getUnitPrice().multiply(v.getQuantity()), BigDecimal::add));

        // 生成结果集
        List<DocumentBySkuVo> documentBySkuVoList = orderSkuList.stream()
                .map(item -> DocumentBySkuVo.builder()
                        .skuSpecId(item.getSkuSpecId())
                        .orderId(item.getOrderId())
                        .quantity(exchangeDetailMap.get(item.getId()))
                        .unitPrice(item.getUnitPrice()
                                .add(packagingMaterialCostMap.getOrDefault(item.getId(), BigDecimal.ZERO))
                                .negate()
                        )
                        .checkFee(BigDecimal.valueOf(2).multiply(exchangeDetailMap.get(item.getId())))
                        .build())
                .peek(item ->
                        item.setSubtotal(item.getQuantity()
                                .multiply(item.getUnitPrice())
                                .add(item.getCheckFee())
                                .setScale(2, RoundingMode.HALF_UP)))
                .peek(item -> item.setTotal(item.getSubtotal()))
                .collect(Collectors.toList());

        // 赋值sku规格品名和品号
        skuSpecService.attributeAssign(documentBySkuVoList, DocumentBySkuVo::getSkuSpecId, (item, skuSpec) -> {
            item.setSkuSpecCode(skuSpec.getCode());
            item.setSkuSpecName(skuSpec.getName());
        });

        // 合并单价、小计、sku规格id系统的对账数据
        Collection<DocumentBySkuVo> documentBySkuVoCollection = documentBySkuVoList.stream()
                .sorted(comparing(DocumentBySkuVo::getSkuSpecCode))
                .collect(Collectors.toMap(
                        item -> item.getUnitPrice() + ":" + item.getSkuSpecId(),
                        Function.identity(),
                        (v1, v2) -> {
                            v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                            v1.setSubtotal(v1.getSubtotal().add(v2.getSubtotal()));
                            v1.setTotal(v1.getTotal().add(v2.getTotal()));
                            return v1;
                        }
                )).values();

        return new ArrayList<>(documentBySkuVoCollection);
    }

    /**
     * 获取可二次利用的包材
     * @param orderSkuBomList
     * @return
     */
    private List<OrderSkuBom> getReusableOrderSkuBomList(List<OrderSkuBom> orderSkuBomList) {
        List<Long> bomSpecIdList = orderSkuBomList.stream()
                .map(OrderSkuBom::getBomSpecId)
                .distinct()
                .collect(Collectors.toList());
        Map<Long, BomSpecBo> bomSpecBoMap = skuSpecService.getBomSpecBoByIdList(bomSpecIdList);
        return orderSkuBomList.stream()
                .filter(item -> {
                    BomSpecBo bomSpecBo = bomSpecBoMap.get(item.getBomSpecId());
                    if (bomSpecBo == null
                            || !(Objects.equals(bomSpecBo.getClassifyId(), BomClassifyConstant.MESH_BAG)
                                || Objects.equals(bomSpecBo.getClassifyId(), BomClassifyConstant.SUSPENDERS))) {
                        return false;
                    }
                    return true;
                }).collect(Collectors.toList());
    }

}
